@@ -5,7 +5,7 @@ use Carp;
 use DBI;
 use base qw(DBI::db);
 
-our $VERSION = '0.02';
+our $VERSION = '0.01';
 
 =head1 NAME
 
@@ -31,8 +31,6 @@ Subclassed DBI connection handle with added convenience features
   # no where clause (i.e. return all rows -  so "select * from $table_name"):
   my @all_employees = database->quick_select('employees', {});
 
-  # count number of male employees
-  my $count = database->quick_count('employees', { gender => 'male' });
 
 =head1 Added features
 
@@ -48,13 +46,6 @@ database handle, with the following added convenience methods:
 Given a table name and a hashref of data (where keys are column names, and the
 values are, well, the values), insert a row in the table.
 
-If you need any of the values to be interpolated straight into the SQL, for
-instance if you need to use a function call like C<NOW()> or similar, then you
-can provide them as a scalarref:
-
-  database->quick_insert('mytable', { foo => 'Bar', timestamp => \'NOW()' });
-
-Of course, if you do that, you must be careful to avoid SQL injection attacks!
 =cut
 
 sub quick_insert {
@@ -69,13 +60,6 @@ sub quick_insert {
 Given a table name, a hashref describing a where clause and a hashref of
 changes, update a row.
 
-As per quick_insert, if you need any of the values to be interpolated straight
-in the SQL, for e.g. to use a function call, provide a scalarref:
-
-  database->quick_update('mytable', { id => 42 }, { counter => \'counter + 1' });
-
-Of course, if you do that, you must be careful to avoid SQL injection attacks!
-
 =cut
 
 sub quick_update {
@@ -158,23 +142,6 @@ An example of using options to control the results you get back:
 
 =cut
 
-=item C<offset> number
-
-C<Offset> says to skip that many rows before beginning to return rows (postgresql).
-
-
-Example:
-
-    # Get the name & phone number of the 10 highest-paid men starting from 11th:
-    database->quick_select(
-        'employees', 
-        { gender => 'male' },
-        { order_by => 'salary', offset => 10, limit => 10, columns => [qw(name phone)] }
-    );
-
-
-=cut
-
 sub quick_select {
     my ($self, $table_name, $where, $opts) = @_;
 
@@ -217,28 +184,6 @@ sub quick_lookup {
     return ( $row && exists $row->{$data} ) ? $row->{$data} : undef;
 }
 
-=item quick_count
-
-  my $count = database->quick_count($table,
-                                    { email => $params->{'email'} });
-
-This is syntactic sugar to return a count of all rows which match your
-parameters, useful for pagination.
-
-This call always returns a single scalar value, not a hashref of the
-entire row (or partial row) like most of the other methods in this
-library.
-
-=cut
-
-sub quick_count {
-    my ($self, $table_name, $where) = @_;
-    my $opts = {}; #Options are irrelevant for a count.
-    my @row = $self->_quick_query('COUNT', $table_name, $opts, $where);
-
-    return ( @row ) ? $row[0] : undef ;
-}
-
 # The 3rd arg, $data, has a different meaning depending on the type of query
 # (no, I don't like that much; I may refactor this soon to use named params).
 # For INSERT/UPDATE queries, it'll be a hashref of field => value.
@@ -247,8 +192,7 @@ sub quick_count {
 sub _quick_query {
     my ($self, $type, $table_name, $data, $where) = @_;
     
-    # Basic sanity checks first...
-    if ($type !~ m{^ (SELECT|INSERT|UPDATE|DELETE|COUNT) $}x) {
+    if ($type !~ m{^ (SELECT|INSERT|UPDATE|DELETE) $}x) {
         carp "Unrecognised query type $type!";
         return;
     }
@@ -262,19 +206,108 @@ sub _quick_query {
         carp "Expected a hashref of changes";
         return;
     }
-    if (($type =~ m{^ (SELECT|UPDATE|DELETE|COUNT) $}x)
+    if (($type =~ m{^ (SELECT|UPDATE|DELETE) $}x)
         && (!$where)) {
         carp "Expected where conditions";
         return;
     }
 
-    # OK, get the SQL we're going to need
-    # TODO: can we replace our own generation with e.g. SQL::Abstract?  How much
-    # backwards-incompatible change would that incur?
-    my ($sql, @bind_params) = $self->_generate_sql(
-        $type, $table_name, $data, $where
-    );
+    my $which_cols = '*';
+    my $opts = $type eq 'SELECT' && $data ? $data : {};
+    if ($opts->{columns}) {
+        my @cols = (ref $opts->{columns}) 
+            ? @{ $opts->{columns} }
+            :    $opts->{columns} ;
+        $which_cols = join(',', map { $self->_quote_identifier($_) } @cols);
+    }
 
+    $table_name = $self->_quote_identifier($table_name);
+    my @bind_params;
+
+    my $sql = {
+        SELECT => "SELECT $which_cols FROM $table_name ",
+        INSERT => "INSERT INTO $table_name ",
+        UPDATE => "UPDATE $table_name SET ",
+        DELETE => "DELETE FROM $table_name ",
+    }->{$type};
+    if ($type eq 'INSERT') {
+        $sql .= "("
+            . join(',', map { $self->_quote_identifier($_) } keys %$data)
+            . ") VALUES ("
+            . join(',', map { "?" } values %$data)
+            . ")";
+        push @bind_params, values %$data;
+    }
+    if ($type eq 'UPDATE') {
+        $sql .= join ',', map { $self->_quote_identifier($_) .'=?' } keys %$data;
+        push @bind_params, values %$data;
+    }
+
+    if ($type eq 'UPDATE' || $type eq 'DELETE' || $type eq 'SELECT') 
+    {
+        if (!ref $where) {
+            $sql .= " WHERE " . $where;
+        }
+        elsif ( ref $where eq 'HASH' ) {
+            my @stmts;
+            foreach my $k ( keys %$where ) {
+                my $v = $where->{$k};
+                if ( ref $v eq 'HASH' ) {
+                    my $not = delete $v->{'not'};
+                    while (my($op,$value) = each %$v ) {
+                        my ($cond, $add_bind_param) 
+                            = $self->_get_where_sql($op, $not, $value);
+                        push @stmts, $self->_quote_identifier($k) . $cond; 
+                        push @bind_params, $v->{$op} if $add_bind_param;
+                    }
+                }
+                else {
+                    my $clause .= $self->_quote_identifier($k);
+                    if ( ! defined $v ) {
+                        $clause .= ' IS NULL';
+                    }
+                    elsif ( ! ref $v ) {
+                        $clause .= '=?';
+                        push @bind_params, $v;
+                    }
+                    elsif ( ref $v eq 'ARRAY' ) {
+                        $clause .= ' IN (' . (join ',', map { '?' } @$v) . ')';
+                        push @bind_params, @$v;
+                    }
+                    push @stmts, $clause;
+                }
+            }
+            $sql .= " WHERE " . join " AND ", @stmts if keys %$where;
+        }
+        else {
+            carp "Can't handle ref " . ref $where . " for where";
+            return;
+        }
+    }
+
+    # Add an ORDER BY clause, if we want to:
+    if (exists $opts->{order_by}) {
+        $sql .= ' ' . $self->_build_order_by_clause($opts->{order_by});
+    }
+
+
+    # Add a LIMIT clause if we want to:
+    if (exists $opts->{limit}) {
+        my $limit = $opts->{limit};
+        $limit =~ s/\s+//g;
+        # Check the limit clause is sane - just a number, or two numbers with a
+        # comma between (if using offset,limit )
+        if ($limit =~ m{ ^ \d+ (?: , \d+)? $ }x) {
+            # Checked for sanity above so safe to interpolate
+            $sql .= " LIMIT $limit";
+        } else {
+            die "Invalid LIMIT param $opts->{limit} !";
+        }
+    } elsif ($type eq 'SELECT' && !wantarray) {
+        # We're only returning one row in scalar context, so don't ask for any
+        # more than that
+        $sql .= " LIMIT 1";
+    }
 
     # Dancer::Plugin::Database will have looked at the log_queries setting and
     # stashed it away for us to see:
@@ -303,140 +336,12 @@ sub _quick_query {
             return $self->selectrow_hashref($sql, undef, @bind_params);
         }
 
-    } elsif ($type eq 'COUNT') {
-        return $self->selectrow_array($sql, undef, @bind_params);
     } else {
         # INSERT/UPDATE/DELETE queries just return the result of DBI's do()
         return $self->do($sql, undef, @bind_params);
     }
 }
 
-sub _generate_sql {
-    my ($self, $type, $table_name, $data, $where) = @_;
-
-    my $which_cols = '*';
-    my $opts = $type eq 'SELECT' && $data ? $data : {};
-    if ($opts->{columns}) {
-        my @cols = (ref $opts->{columns}) 
-            ? @{ $opts->{columns} }
-            :    $opts->{columns} ;
-        $which_cols = join(',', map { $self->_quote_identifier($_) } @cols);
-    }
-
-    $table_name = $self->_quote_identifier($table_name);
-    my @bind_params;
-
-    my $sql = {
-        SELECT => "SELECT $which_cols FROM $table_name",
-        INSERT => "INSERT INTO $table_name ",
-        UPDATE => "UPDATE $table_name SET ",
-        DELETE => "DELETE FROM $table_name ",
-        COUNT => "SELECT COUNT(*) FROM $table_name",
-    }->{$type};
-    if ($type eq 'INSERT') {
-        my (@keys, @values);
-        for my $key (sort keys %$data) {
-            my $value = $data->{$key};
-            push @keys, $self->_quote_identifier($key);
-            if (ref $value eq 'SCALAR') {
-                # If it's a scalarref it goes in the SQL as it is; this is a
-                # potential SQL injection risk, but is documented as such - it
-                # allows the user to include arbitrary SQL, at their own risk.
-                push @values, $$value;
-            } else {
-                push @values, "?";
-                push @bind_params, $value;
-            }
-        }
-
-        $sql .= sprintf "(%s) VALUES (%s)",
-            join(',', @keys), join(',', @values);
-    }
-
-    if ($type eq 'UPDATE') {
-        my (@keys, @values);
-        $sql .= join ',', map {
-            $self->_quote_identifier($_) .'=' 
-            . (ref $data->{$_} eq 'SCALAR' ? ${$data->{$_}} : "?")
-        } sort keys %$data;
-        push @bind_params, grep { ref $_ ne 'SCALAR' } values %$data;
-    }
-
-    if ($type eq 'UPDATE' || $type eq 'DELETE' || $type eq 'SELECT' || $type eq 'COUNT')
-    {
-        if ($where && !ref $where) {
-            $sql .= " WHERE " . $where;
-        } elsif ( ref $where eq 'HASH' ) {
-            my @stmts;
-            foreach my $k ( sort keys %$where ) {
-                my $v = $where->{$k};
-                if ( ref $v eq 'HASH' ) {
-                    my $not = delete $v->{'not'};
-                    while (my($op,$value) = each %$v ) {
-                        my ($cond, $add_bind_param) 
-                            = $self->_get_where_sql($op, $not, $value);
-                        push @stmts, $self->_quote_identifier($k) . $cond; 
-                        push @bind_params, $v->{$op} if $add_bind_param;
-                    }
-                } else {
-                    my $clause .= $self->_quote_identifier($k);
-                    if ( ! defined $v ) {
-                        $clause .= ' IS NULL';
-                    }
-                    elsif ( ! ref $v ) {
-                        $clause .= '=?';
-                        push @bind_params, $v;
-                    }
-                    elsif ( ref $v eq 'ARRAY' ) {
-                        $clause .= ' IN (' . (join ',', map { '?' } @$v) . ')';
-                        push @bind_params, @$v;
-                    }
-                    push @stmts, $clause;
-                }
-            }
-            $sql .= " WHERE " . join " AND ", @stmts if keys %$where;
-        } elsif (ref $where) {
-            carp "Can't handle ref " . ref $where . " for where";
-            return;
-        }
-    }
-
-    # Add an ORDER BY clause, if we want to:
-    if (exists $opts->{order_by} and defined $opts->{order_by}) {
-        $sql .= ' ' . $self->_build_order_by_clause($opts->{order_by});
-    }
-
-
-    # Add a LIMIT clause if we want to:
-    if (exists $opts->{limit} and defined $opts->{limit}) {
-        my $limit = $opts->{limit};
-        $limit =~ s/\s+//g;
-        # Check the limit clause is sane - just a number, or two numbers with a
-        # comma between (if using offset,limit )
-        if ($limit =~ m{ ^ \d+ (?: , \d+)? $ }x) {
-            # Checked for sanity above so safe to interpolate
-            $sql .= " LIMIT $limit";
-        } else {
-            die "Invalid LIMIT param $opts->{limit} !";
-        }
-    } elsif ($type eq 'SELECT' && !wantarray) {
-        # We're only returning one row in scalar context, so don't ask for any
-        # more than that
-        $sql .= " LIMIT 1";
-    }
-    
-    if (exists $opts->{offset} and defined $opts->{offset}) {
-        my $offset = $opts->{offset};
-        $offset =~ s/\s+//g;
-        if ($offset =~ /^\d+$/) {
-            $sql .= " OFFSET $offset";
-        } else {
-            die "Invalid OFFSET param $opts->{offset} !";
-        }
-    }
-    return ($sql, @bind_params);
-}
-
 sub _get_where_sql {
     my ($self, $op, $not, $value) = @_;
 
@@ -582,7 +487,7 @@ Currently recognized operators are:
 ... same as C<WHERE foo LIKE '%bar%'>
 
 =item 'gt' / 'ge'
-
+ 
  'greater than' or 'greater or equal to'
   
  { foo => { 'ge' => '42' } } 
@@ -596,7 +501,7 @@ Currently recognized operators are:
  { foo => { 'lt' => '42' } } 
 
 ... same as C<WHERE foo E<lt> '42'>
-
+ 
 =item 'eq' / 'ne' / 'is'
 
  'equal' or 'not equal' or 'is'
